@@ -27,6 +27,7 @@
 #include "gstprops.h"
 
 GType _gst_props_type;
+GType _gst_props_entry_type;
 
 #define GST_PROPS_ENTRY_IS_VARIABLE(a)	(((GstPropsEntry*)(a))->propstype > GST_PROPS_VAR_TYPE)
 
@@ -83,12 +84,12 @@ transform_func (const GValue *src_value,
       const gchar *name = g_quark_to_string (entry->propid);
 
       switch (entry->propstype) {
-        case GST_PROPS_STRING_TYPE:
-	  g_string_append_printf (result, "%s=(string) '%s'", name, entry->data.string_data.string);
-	  break;
         case GST_PROPS_INT_TYPE:
   	  g_string_append_printf (result, "%s=(int) %d", name, entry->data.int_data);
 	  break;
+        case GST_PROPS_FLOAT_TYPE:
+  	  g_string_append_printf (result, "%s=(float) %f", name, entry->data.float_data);
+	  break;
         case GST_PROPS_FOURCC_TYPE:
   	  g_string_append_printf (result, "%s=(fourcc) '%4.4s'", name, (gchar *)&entry->data.fourcc_data);
 	  break;
@@ -96,6 +97,9 @@ transform_func (const GValue *src_value,
   	  g_string_append_printf (result, "%s=(boolean) %s", name, 
 			  (entry->data.bool_data ? "TRUE" : "FALSE"));
 	  break;
+        case GST_PROPS_STRING_TYPE:
+	  g_string_append_printf (result, "%s=(string) '%s'", name, entry->data.string_data.string);
+	  break;
         default:
 	  break;
       }
@@ -107,6 +111,7 @@ transform_func (const GValue *src_value,
     }
   }
   dest_value->data[0].v_pointer = result->str;
+  g_string_free (result, FALSE);
 }
 
 	
@@ -130,6 +135,10 @@ _gst_props_initialize (void)
   g_value_register_transform_func (_gst_props_type,
                                    G_TYPE_STRING,
                                    transform_func);
+
+  _gst_props_entry_type = g_boxed_type_register_static ("GstPropsEntry",
+		  (GBoxedCopyFunc) gst_props_entry_copy,
+		  (GBoxedFreeFunc) gst_props_entry_destroy);
 }
 
 static void
@@ -301,7 +310,7 @@ gst_props_alloc_entry (void)
   return entry;
 }
 
-static void
+void
 gst_props_entry_destroy (GstPropsEntry *entry)
 {
   switch (entry->propstype) {
@@ -345,11 +354,45 @@ gst_props_empty_new (void)
 
   props->properties = NULL;
   props->refcount = 1;
-  props->fixed = TRUE;
+  GST_PROPS_FLAG_SET (props, GST_PROPS_FLOATING);
+  GST_PROPS_FLAG_SET (props, GST_PROPS_FIXED);
 
   return props;
 }
 
+/**
+ * gst_props_replace:
+ * @oldprops: the props to take replace
+ * @newprops: the props to take replace 
+ *
+ * Replace the pointer to the props, doing proper
+ * refcounting.
+ */
+void
+gst_props_replace (GstProps **oldprops, GstProps *newprops)
+{
+  if (*oldprops != newprops) {
+    if (newprops)  gst_props_ref   (newprops);
+    if (*oldprops) gst_props_unref (*oldprops);
+
+    *oldprops = newprops;
+  }
+}
+
+/**
+ * gst_props_replace_sink:
+ * @oldprops: the props to take replace
+ * @newprops: the props to take replace 
+ *
+ * Replace the pointer to the proppropsd take ownership.
+ */
+void
+gst_props_replace_sink (GstProps **oldprops, GstProps *newprops)
+{
+  gst_props_replace (oldprops, newprops);
+  gst_props_sink (newprops);
+}
+
 /**
  * gst_props_add_entry:
  * @props: the property to add the entry to
@@ -363,8 +406,8 @@ gst_props_add_entry (GstProps *props, GstPropsEntry *entry)
   g_return_if_fail (props);
   g_return_if_fail (entry);
 
-  if (props->fixed && GST_PROPS_ENTRY_IS_VARIABLE (entry)) {
-    props->fixed = FALSE;
+  if (GST_PROPS_IS_FIXED (props) && GST_PROPS_ENTRY_IS_VARIABLE (entry)) {
+    GST_PROPS_FLAG_UNSET (props, GST_PROPS_FIXED);
   }
   props->properties = g_list_insert_sorted (props->properties, entry, props_compare_func);
 }
@@ -732,17 +775,23 @@ gst_props_set (GstProps *props, const gchar *name, ...)
  *
  * Decrease the refcount of the property structure, destroying
  * the property if the refcount is 0.
+ *
+ * Returns: refcounted GstProps or NULL if props was destroyed.
  */
-void
+GstProps*
 gst_props_unref (GstProps *props)
 {
   if (props == NULL)
-    return;
+    return NULL;
   
   props->refcount--;
 
-  if (props->refcount == 0)
+  if (props->refcount == 0) {
     gst_props_destroy (props);
+    return NULL;
+  }
+
+  return props;
 }
 
 /**
@@ -750,15 +799,38 @@ gst_props_unref (GstProps *props)
  * @props: the props to ref
  *
  * Increase the refcount of the property structure.
+ *
+ * Returns: refcounted GstProps.
  */
-void
+GstProps*
 gst_props_ref (GstProps *props)
 {
-  g_return_if_fail (props != NULL);
+  if (props == NULL)
+    return NULL;
   
   props->refcount++;
+
+  return props;
 }
 
+/**
+ * gst_props_sink:
+ * @props: the props to sink
+ *
+ * If the props if floating, decrease its refcount. Usually used 
+ * with gst_props_ref() to take ownership of the props.
+ */
+void
+gst_props_sink (GstProps *props)
+{
+  if (props == NULL)
+    return;
+
+  if (GST_PROPS_IS_FLOATING (props)) {
+    GST_PROPS_FLAG_UNSET (props, GST_PROPS_FLOATING);
+    gst_props_unref (props);
+  }
+}
 
 /**
  * gst_props_destroy:
@@ -791,18 +863,24 @@ gst_props_destroy (GstProps *props)
 /* 
  * copy entries 
  */
-static GstPropsEntry*
+GstPropsEntry*
 gst_props_entry_copy (GstPropsEntry *entry)
 {
   GstPropsEntry *newentry;
 
   newentry = gst_props_alloc_entry ();
   memcpy (newentry, entry, sizeof (GstPropsEntry));
-  if (entry->propstype == GST_PROPS_LIST_TYPE) {
-    newentry->data.list_data.entries = gst_props_list_copy (entry->data.list_data.entries);
-  }
-  else if (entry->propstype == GST_PROPS_STRING_TYPE) {
-    newentry->data.string_data.string = g_strdup (entry->data.string_data.string);
+
+  switch (entry->propstype) {
+    case GST_PROPS_LIST_TYPE:
+      newentry->data.list_data.entries = gst_props_list_copy (entry->data.list_data.entries);
+      break;
+    case GST_PROPS_STRING_TYPE:
+      newentry->data.string_data.string = g_strdup (entry->data.string_data.string);
+      break;
+    default:
+      /* FIXME more? */
+      break;
   }
 
   return newentry;
@@ -844,7 +922,8 @@ gst_props_copy (GstProps *props)
 
   new = gst_props_empty_new ();
   new->properties = gst_props_list_copy (props->properties);
-  new->fixed = props->fixed;
+  /* copy fixed and set floating flags */
+  GST_PROPS_FLAGS (new) = GST_PROPS_FLAGS (props) | GST_PROPS_FLOATING;
 
   return new;
 }
@@ -1718,7 +1797,7 @@ gst_props_intersect (GstProps *props1, GstProps *props2)
   GstPropsEntry *iprops = NULL;
 
   intersection = gst_props_empty_new ();
-  intersection->fixed = TRUE;
+  GST_PROPS_FLAG_SET (intersection, GST_PROPS_FIXED);
 
   g_return_val_if_fail (props1 != NULL, NULL);
   g_return_val_if_fail (props2 != NULL, NULL);
@@ -1743,7 +1822,7 @@ gst_props_intersect (GstProps *props1, GstProps *props2)
 
       toadd = gst_props_entry_copy (entry1);
       if (GST_PROPS_ENTRY_IS_VARIABLE (toadd))
-	intersection->fixed = FALSE;
+        GST_PROPS_FLAG_UNSET (intersection, GST_PROPS_FIXED);
 
       intersection->properties = g_list_prepend (intersection->properties, toadd);
 
@@ -1758,7 +1837,7 @@ gst_props_intersect (GstProps *props1, GstProps *props2)
 
       toadd = gst_props_entry_copy (entry2);
       if (GST_PROPS_ENTRY_IS_VARIABLE (toadd))
-	intersection->fixed = FALSE;
+        GST_PROPS_FLAG_UNSET (intersection, GST_PROPS_FIXED);
 
       intersection->properties = g_list_prepend (intersection->properties, toadd);
 
@@ -1773,7 +1852,8 @@ gst_props_intersect (GstProps *props1, GstProps *props2)
 
     if (iprops) {
       if (GST_PROPS_ENTRY_IS_VARIABLE (iprops))
-	intersection->fixed = FALSE;
+        GST_PROPS_FLAG_UNSET (intersection, GST_PROPS_FIXED);
+
       intersection->properties = g_list_prepend (intersection->properties, iprops);
     }
     else {
@@ -1799,7 +1879,8 @@ end:
 
     entry = (GstPropsEntry *) leftovers->data;
     if (GST_PROPS_ENTRY_IS_VARIABLE (entry))
-      intersection->fixed = FALSE;
+      GST_PROPS_FLAG_UNSET (intersection, GST_PROPS_FIXED);
+
     intersection->properties = g_list_prepend (intersection->properties, gst_props_entry_copy (entry));
 
     leftovers = g_list_next (leftovers);
