@@ -113,7 +113,7 @@ print_caps (const GstCaps *caps, const gchar *pfx)
   while (caps) {
     GstType *type;
 
-    g_print ("%s'%s': (%sfixed)\n", pfx, caps->name, (caps->fixed ? "" : "NOT "));
+    g_print ("%s'%s': (%sfixed)\n", pfx, caps->name, (GST_CAPS_IS_FIXED (caps) ? "" : "NOT "));
 
     type = gst_type_find_by_id (caps->id);
     if (type) 
@@ -150,6 +150,23 @@ print_formats (const GstFormat *formats)
   }
 }
 
+static void 
+print_query_types (const GstQueryType *types) 
+{
+  while (types && *types) {
+    const GstQueryTypeDefinition *definition;
+
+    definition = gst_query_type_get_details (*types);
+    if (definition)
+      g_print ("\t\t(%d):\t%s (%s)\n", *types,
+	       definition->nick, definition->description);
+    else
+      g_print ("\t\t(%d):\tUnknown query format\n", *types);
+
+    types++;
+  }
+}
+
 static void 
 print_event_masks (const GstEventMask *masks) 
 {
@@ -198,26 +215,6 @@ print_event_masks (const GstEventMask *masks)
   }
 }
 
-static void 
-print_query_types (const GstQueryType *types) 
-{
-  GType query_type;
-  GEnumClass *klass;
-
-  query_type = gst_query_type_get_type();
-  klass = (GEnumClass *) g_type_class_ref (query_type);
-
-  while (types && *types) {
-    GEnumValue *value;
-    
-    value = g_enum_get_value (klass, *types);
-    
-    g_print ("\t\t(%d):\t%s (%s)\n", value->value, value->value_nick, value->value_name);
-    types++;
-  }
-}
-
-
 static void
 output_hierarchy (GType type, gint level, gint *maxlevel)
 {
@@ -314,7 +311,7 @@ print_element_properties (GstElement *element)
       {
 	GParamSpecUInt64 *puint64 = G_PARAM_SPEC_UINT64 (param);
 	g_print("%-23.23s Unsigned Integer64. ", ""); 
-	if (readable) g_print("Range: %llu - %llu (Default %llu)", 
+	if (readable) g_print("Range: %" G_GUINT64_FORMAT " - %" G_GUINT64_FORMAT " (Default %" G_GUINT64_FORMAT ")", 
 			puint64->minimum, puint64->maximum, g_value_get_uint64 (&value));
 	break;
       }
@@ -322,7 +319,7 @@ print_element_properties (GstElement *element)
       {
 	GParamSpecInt64 *pint64 = G_PARAM_SPEC_INT64 (param);
 	g_print("%-23.23s Integer64. ", ""); 
-	if (readable) g_print("Range: %lld - %lld (Default %lld)", 
+	if (readable) g_print("Range: %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT " (Default %" G_GINT64_FORMAT ")", 
 			pint64->minimum, pint64->maximum, g_value_get_int64 (&value));
 	break;
       }
@@ -429,7 +426,7 @@ print_element_properties (GstElement *element)
         break;
     }
     if (!readable) 
-      g_print ("Write only\n");
+      g_print (" Write only\n");
     else 
       g_print ("\n");
   }
@@ -647,9 +644,9 @@ print_element_info (GstElementFactory *factory)
 	print_query_types (gst_pad_get_query_types (GST_PAD (realpad)));
       }
 
-      if (realpad->intconnfunc != gst_pad_get_internal_connections_default)
+      if (realpad->intlinkfunc != gst_pad_get_internal_links_default)
         g_print ("      Has custom intconnfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME(realpad->intconnfunc));
+	        GST_DEBUG_FUNCPTR_NAME(realpad->intlinkfunc));
 
       if (realpad->bufferpoolfunc)
         g_print ("      Has bufferpoolfunc(): %s\n",
@@ -683,7 +680,7 @@ print_element_info (GstElementFactory *factory)
 
         switch (G_PARAM_SPEC_VALUE_TYPE (specs[x])) {
           case G_TYPE_INT64: 
-            g_print ("64 Bit Integer (Default %lld, Range %lld -> %lld)", 
+            g_print ("64 Bit Integer (Default %" G_GINT64_FORMAT ", Range %" G_GINT64_FORMAT " -> %" G_GINT64_FORMAT ")", 
             ((GParamSpecInt64 *) specs[x])->default_value,
             ((GParamSpecInt64 *) specs[x])->minimum, 
             ((GParamSpecInt64 *) specs[x])->maximum);
@@ -842,6 +839,14 @@ print_element_list (void)
         g_print ("%s:  %s: %s\n", plugin->name, 
 	        GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
+      else if (GST_IS_URI_HANDLER (feature)) {
+        GstURIHandler *handler;
+
+        handler = GST_URI_HANDLER (feature);
+        g_print ("%s:  %s: \"%s\" (%s) element \"%s\" property \"%s\"\n", plugin->name, 
+	        GST_PLUGIN_FEATURE_NAME (handler), handler->uri, handler->longdesc,
+		handler->element, handler->property);
+      }
       else {
         g_print ("%s:  %s (%s)\n", plugin->name, 
 	        GST_PLUGIN_FEATURE_NAME (feature), 
@@ -983,6 +988,36 @@ main (int argc, char *argv[])
       /* if there's a factory, print out the info */
       if (factory)
         return print_element_info (factory);
+      else {
+	 GstPluginFeature* feature;
+
+	 /* FIXME implement other pretty print function for these */
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_SCHEDULER_FACTORY);
+	 if (feature) {
+           g_print ("%s: a scheduler\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_INDEX_FACTORY);
+	 if (feature) {
+           g_print ("%s: an index\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_AUTOPLUG_FACTORY);
+	 if (feature) {
+           g_print ("%s: an autoplugger\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_TYPE_FACTORY);
+	 if (feature) {
+           g_print ("%s: an type\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_URI_HANDLER);
+	 if (feature) {
+           g_print ("%s: an uri handler\n", argv[1]);
+	   return 0;
+	 }
+      }
     } else {
       /* strip the .so */
       so = strstr(argv[1],".so");
